from typing import Any, Dict, List, cast

from typing_extensions import NotRequired, TypedDict

from resend import request
from resend.domains._domain import Domain


class Domains:
    class UpdateParams(TypedDict):
        id: str
        """
        The domain ID.
        """
        click_tracking: NotRequired[bool]
        """
        Track clicks within the body of each HTML email.
        """
        open_tracking: NotRequired[bool]
        """
        Track the open rate of each email.
        """

    class CreateParams(TypedDict):
        name: str
        """
        The domain name.
        """
        region: NotRequired[str]
        """
        The region where emails will be sent from.
        Possible values: us-east-1' | 'eu-west-1' | 'sa-east-1' | 'ap-northeast-1'
        """

    @classmethod
    def create(cls, params: CreateParams) -> Domain:
        """
        Create a domain through the Resend Email API.
        see more: https://resend.com/docs/api-reference/domains/create-domain

        Args:
            params (CreateParams): The domain creation parameters

        Returns:
            Domain: The new domain object
        """
        path = "/domains"
        return Domain.new_from_request(
            request.Request(
                path=path, params=cast(Dict[Any, Any], params), verb="post"
            ).perform()
        )

    @classmethod
    def update(cls, params: UpdateParams) -> Domain:
        """
        Update an existing domain.
        see more: https://resend.com/docs/api-reference/domains/update-domain

        Args:
            params (UpdateParams): The domain update parameters

        Returns:
            Domain: The updated domain object
        """
        path = f"/domains/{params['id']}"
        return Domain.new_from_request(
            request.Request(
                path=path, params=cast(Dict[Any, Any], params), verb="patch"
            ).perform()
        )

    @classmethod
    def get(cls, domain_id: str) -> Domain:
        """
        Retrieve a single domain for the authenticated user.
        see more: https://resend.com/docs/api-reference/domains/get-domain

        Args:
            domain_id (str): The domain ID

        Returns:
            Domain: The domain object
        """
        path = f"/domains/{domain_id}"
        return Domain.new_from_request(
            request.Request(path=path, params={}, verb="get").perform()
        )

    @classmethod
    def list(cls) -> List[Domain]:
        """
        Retrieve a list of domains for the authenticated user.
        see more: https://resend.com/docs/api-reference/domains/list-domains

        Returns:
            List[Domain]: A list of domain objects
        """
        path = "/domains"
        resp = request.Request(path=path, params={}, verb="get").perform()
        return [Domain.new_from_request(val) for val in resp["data"]]

    @classmethod
    def remove(cls, domain_id: str) -> Domain:
        """
        Remove an existing domain.
        see more: https://resend.com/docs/api-reference/domains/delete-domain

        Args:
            domain_id (str): The domain ID

        Returns:
            Domain: The removed domain object
        """
        path = f"/domains/{domain_id}"
        return Domain.new_from_request(
            request.Request(path=path, params={}, verb="delete").perform()
        )

    @classmethod
    def verify(cls, domain_id: str) -> Domain:
        """
        Verify an existing domain.
        see more: https://resend.com/docs/api-reference/domains/verify-domain

        Args:
            domain_id (str): The domain ID

        Returns:
            Domain: The verified domain object
        """
        path = f"/domains/{domain_id}/verify"
        return Domain.new_from_request(
            request.Request(path=path, params={}, verb="post").perform()
        )



from typing import Any, Dict


class Contact:
    id: str
    """
    The contact id.
    """
    email: str
    """
    The email of the contact.
    """
    first_name: str
    """
    The first name of the contact.
    """
    last_name: str
    """
    The last name of the contact.
    """
    created_at: str
    """
    The timestamp of the contact.
    """
    unsubscribed: bool
    """
    The unsubscribed status of the contact.
    """
    deleted: bool
    """
    Wether the contact is deleted or not.
    """

    def __init__(
        self,
        id: str,
        email: str,
        first_name: str,
        last_name: str,
        created_at: str,
        unsubscribed: bool,
        deleted: bool,
    ):
        self.id = id
        self.email = email
        self.first_name = first_name
        self.last_name = last_name
        self.created_at = created_at
        self.unsubscribed = unsubscribed
        self.deleted = deleted

    @staticmethod
    def new_from_request(val: Dict[Any, Any]) -> "Contact":
        """Creates a new Contact object from the
        JSON response from the API.

        Args:
            val (Dict): The JSON response from the API

        Returns:
            Contact: The new Contact object
        """
        contact = Contact(
            id=val["id"] if "id" in val else "",
            email=val["email"] if "email" in val else "",
            first_name=val["first_name"] if "first_name" in val else "",
            last_name=val["last_name"] if "last_name" in val else "",
            created_at=val["created_at"] if "created_at" in val else "",
            unsubscribed=val["unsubscribed"] if "unsubscribed" in val else False,
            deleted=val["deleted"] if "deleted" in val else False,
        )
        return contact



from typing import List, Union

from typing_extensions import NotRequired, TypedDict


class Attachment(TypedDict):
    content: Union[List[int], str]
    """
    Content of an attached file.
    This is a list of integers which is usually translated from a
    "bytes" type, OR a string
    Ie: list(open("file.pdf", "rb").read())
    """
    filename: str
    """
    Name of attached file.
    """
    path: NotRequired[str]
    """
    Path where the attachment file is hosted
    """



from typing import Any, Dict, List, Union

from resend.domains._record import Record


class Domain:
    id: str
    """
    The domain ID
    """
    name: str
    """
    The domain name
    """
    created_at: str
    """
    When domain was created
    """
    status: str
    """
    Status of the domain: not_started, etc..
    """
    region: str
    """
    The region where emails will be sent from. Possible values: us-east-1' | 'eu-west-1' | 'sa-east-1' | 'ap-northeast-1'
    """
    records: Union[List[Record], None]
    """
    The list of domain records
    """
    deleted: bool
    """
    Wether the domain is deleted or not
    """

    def __init__(
        self,
        id: str,
        name: str,
        region: str,
        created_at: str,
        status: str,
        records: Union[List[Record], None],
        deleted: bool = False,
    ):
        self.id = id
        self.name = name
        self.region = region
        self.created_at = created_at
        self.status = status
        self.records = records
        self.deleted = deleted

    @staticmethod
    def new_from_request(val: Dict[Any, Any]) -> "Domain":
        """Creates a new Domain object from the
        JSON response from the API.

        Args:
            val (Dict): The JSON response from the API

        Returns:
            Domain: The new Domain object
        """
        domain = Domain(
            id=val["id"] if "id" in val else None,
            name=val["name"] if "name" in val else None,
            region=val["region"] if "region" in val else None,
            created_at=val["created_at"] if "created_at" in val else None,
            status=val["status"] if "status" in val else None,
            records=(
                [Record.new_from_request(record) for record in val["records"]]
                if "records" in val
                else None
            ),
            deleted=val["deleted"] if "deleted" in val else None,
        )
        return domain



from typing import Any, Dict


class Audience:
    id: str
    """
    The unique identifier of the audience.
    """
    name: str
    """
    The name of the audience.
    """
    created_at: str
    """
    The date and time the audience was created.
    """
    deleted: bool
    """
    Wether the audience was deleted. Only returned on the "remove" call
    """

    def __init__(self, id: str, name: str, created_at: str, deleted: bool = False):
        self.id = id
        self.name = name
        self.created_at = created_at
        self.deleted = deleted

    @staticmethod
    def new_from_request(val: Dict[Any, Any]) -> "Audience":
        """Creates a new Audience object from the
        JSON response from the API.

        Args:
            val (Dict): The JSON response from the API

        Returns:
            Audience: The new Audience object
        """
        audience = Audience(
            id=val["id"] if "id" in val else None,
            name=val["name"] if "name" in val else None,
            created_at=val["created_at"] if "created_at" in val else None,
            deleted=val["deleted"] if "deleted" in val else False,
        )
        return audience



from typing import Any, Dict


class Record:
    record: str
    """
    The domain record type, ie: SPF.
    """
    name: str
    """
    The domain record name.
    """
    type: str
    """
    The domain record type, ie: MX.
    """
    ttl: str
    """
    The domain record time to live.
    """
    status: str
    """
    The domain record status: not_started, etc..
    """
    value: str
    """
    The domain record value.
    """
    priority: int
    """
    The domain record priority.
    """

    def __init__(
        self,
        record: str,
        name: str,
        type: str,
        ttl: str,
        status: str,
        value: str,
        priority: int,
    ):
        self.record = record
        self.name = name
        self.type = type
        self.ttl = ttl
        self.status = status
        self.value = value
        self.priority = priority

    @staticmethod
    def new_from_request(val: Dict[Any, Any]) -> "Record":
        return Record(
            record=val["record"] if "record" in val else None,
            name=val["name"] if "name" in val else None,
            type=val["type"] if "type" in val else None,
            ttl=val["ttl"] if "ttl" in val else None,
            status=val["status"] if "status" in val else None,
            value=val["value"] if "value" in val else None,
            priority=val["priority"] if "priority" in val else None,
        )



from typing import Dict

import requests

import resend
from resend.exceptions import raise_for_code_and_type
from resend.version import get_version


# This class wraps the HTTP request creation logic
class Request:
    def __init__(self, path: str, params: Dict, verb: str):
        self.path = path
        self.params = params
        self.verb = verb

    def perform(self):
        """Is the main function that makes the HTTP request
        to the Resend API. It uses the path, params, and verb attributes
        to make the request.

        Returns:
            Dict: The JSON response from the API

        Raises:
            requests.HTTPError: If the request fails
        """
        resp = self.make_request(url=f"{resend.api_url}{self.path}")

        # delete calls do not return a body
        if resp.text == "" and resp.status_code == 200:
            return None

        # handle error in case there is a statusCode attr present
        # and status != 200
        if resp.status_code != 200 and resp.json().get("statusCode"):
            error = resp.json()
            raise_for_code_and_type(
                code=error.get("statusCode"),
                message=error.get("message"),
                error_type=error.get("name"),
            )
        return resp.json()

    def __get_headers(self) -> Dict:
        """get_headers returns the HTTP headers that will be
        used for every req.

        Returns:
            Dict: configured HTTP Headers
        """
        return {
            "Accept": "application/json",
            "Authorization": f"Bearer {resend.api_key}",
            "User-Agent": f"resend-python:{get_version()}",
        }

    def make_request(self, url: str) -> requests.Response:
        """make_request is a helper function that makes the actual
        HTTP request to the Resend API.

        Args:
            url (str): The URL to make the request to

        Returns:
            requests.Response: The response object from the request

        Raises:
            requests.HTTPError: If the request fails
        """
        headers = self.__get_headers()
        params = self.params
        verb = self.verb

        try:
            return requests.request(verb, url, json=params, headers=headers)
        except requests.HTTPError as e:
            raise e



from typing import Any, Dict, List, Union, cast

from typing_extensions import NotRequired, TypedDict

from resend import request
from resend.emails._attachment import Attachment
from resend.emails._email import Email
from resend.emails._tag import Tag


class Emails:
    class SendParams(TypedDict):
        sender: str
        """
        The email address of the sender.
        "from" is a reserved keyword in python, so we use "sender" here instead
        """
        to: Union[str, List[str]]
        """
        List of email addresses to send the email to.
        """
        subject: str
        """
        The subject of the email.
        """
        bcc: NotRequired[Union[List[str], str]]
        """
        Bcc
        """
        cc: NotRequired[Union[List[str], str]]
        """
        Cc
        """
        reply_to: NotRequired[Union[List[str], str]]
        """
        Reply to
        """
        html: NotRequired[str]
        """
        The HTML content of the email.
        """
        text: NotRequired[str]
        """
        The text content of the email.
        """
        headers: NotRequired[Dict[str, str]]
        """
        Custom headers to be added to the email.
        """
        attachments: NotRequired[List[Attachment]]
        """
        List of attachments to be added to the email.
        """
        tags: NotRequired[List[Tag]]
        """
        List of tags to be added to the email.
        """

    @classmethod
    def send(cls, params: SendParams) -> Email:
        """
        Send an email through the Resend Email API.
        see more: https://resend.com/docs/api-reference/emails/send-email

        Args:
            params (SendParams): The email parameters

        Returns:
            Email: The email object that was sent
        """
        path = "/emails"

        # we need this workaround here because from is a reserved keyword
        # in python, so we need to use "sender" on the SendParams
        params["from"] = params["sender"]  # type: ignore
        return Email.new_from_request(
            request.Request(
                path=path, params=cast(Dict[Any, Any], params), verb="post"
            ).perform()
        )

    @classmethod
    def get(cls, email_id: str) -> Email:
        """
        Retrieve a single email.
        see more: https://resend.com/docs/api-reference/emails/retrieve-email

        Args:
            email_id (str): The ID of the email to retrieve

        Returns:
            Email: The email object that was retrieved
        """
        path = f"/emails/{email_id}"
        return Email.new_from_request(
            request.Request(path=path, params={}, verb="get").perform()
        )



from typing import Any, Dict, List, cast

from typing_extensions import TypedDict

from resend import request

from ._audience import Audience


class Audiences:
    class CreateParams(TypedDict):
        name: str
        """
        The name of the audience.
        """

    @classmethod
    def create(cls, params: CreateParams) -> Audience:
        """
        Create a list of contacts.
        see more: https://resend.com/docs/api-reference/audiences/create-audience

        Args:
            params (CreateParams): The audience creation parameters

        Returns:
            Audience: The new audience object
        """
        path = "/audiences"
        return Audience.new_from_request(
            request.Request(
                path=path, params=cast(Dict[Any, Any], params), verb="post"
            ).perform()
        )

    @classmethod
    def list(cls) -> List[Audience]:
        """
        Retrieve a list of audiences.
        see more: https://resend.com/docs/api-reference/audiences/list-audiences

        Returns:
            List[Audience]: A list of audience objects
        """
        path = "/audiences/"
        resp = request.Request(path=path, params={}, verb="get").perform()
        return (
            [Audience.new_from_request(aud) for aud in resp["data"]]
            if "data" in resp
            else []
        )

    @classmethod
    def get(cls, id: str) -> Audience:
        """
        Retrieve a single audience.
        see more: https://resend.com/docs/api-reference/audiences/get-audience

        Args:
            id (str): The audience ID

        Returns:
            Audience: The audience object
        """
        path = f"/audiences/{id}"
        return Audience.new_from_request(
            request.Request(path=path, params={}, verb="get").perform()
        )

    @classmethod
    def remove(cls, id: str) -> Audience:
        """
        Delete a single audience.
        see more: https://resend.com/docs/api-reference/audiences/delete-audience

        Args:
            id (str): The audience ID

        Returns:
            Audience: The audience object
        """
        path = f"/audiences/{id}"
        return Audience.new_from_request(
            request.Request(path=path, params={}, verb="delete").perform()
        )



from typing import Any, Dict


class ApiKey:
    id: str
    """
    The api key ID
    """
    token: str
    """
    The api key token
    """
    name: str
    """
    The api key token
    """
    created_at: str
    """
    Api key creation date
    """

    def __init__(self, id: str, token: str, name: str, created_at: str):
        self.id = id
        self.token = token
        self.name = name
        self.created_at = created_at

    @staticmethod
    def new_from_request(val: Dict[Any, Any]) -> "ApiKey":
        """Creates a new ApiKey object from the
        JSON response from the API.

        Args:
            val (Dict): The JSON response from the API

        Returns:
            ApiKey: The new ApiKey object
        """
        return ApiKey(
            id=val["id"],
            token=val["token"] if "token" in val else None,
            name=val["name"] if "name" in val else None,
            created_at=val["created_at"] if "created_at" in val else None,
        )
